"""One of the best ways to store a graph in python is to use a dictionary
object.

g = {}

Nodes in a graph are the keys in a dictionary and the edges are the values.
To traverse through the nodes in a graph

for n in g:
   # do something with the node

to traverse through key and edge values

for n,e in g.items()

to traverse through edge values
for n in g.values()

"""
from types import DictType

class Graph(DictType):
    def nodes(self):
        return self.keys()

    def edges(self):
        _edges = {}
        for neighbor in self.values():
            for edge in neighbor.values():
                _edges[edge] = 1

        return _edges.keys()
    
    def iternodes(self):
        """iterate through the nodes"""
        return self.iterkeys()

    def neighbors(self, node):
        """(node) -> return the (node, edge) pairs for a given
        node"""
        return self[node].items()

    def iterneighbors(self):
        """iterate through the neighbors of a node"""
        return self.iteritems()

    def addNode(self, node):
        self[node] = self.get(node, {})
        node.setParent(self)
        
    def addEdge(self, n1, n2, edge):
        # assumes n1 and n2 are already in a node
        try:
            assert not self[n1].has_key(n2)
            assert not self[n2].has_key(n1)
            self[n1][n2] = edge
            self[n2][n1] = edge
        except KeyError:
            raise GraphError("One of the edge nodes is not in the graph")

from weakref import proxy

class Base:
    def __init__(self, label):
        self.label = label
        self.equiv_class = -1
        self.symorder = -1
        self.symclass = -1
        self.parent = None

    def setParent(self, parent):
        self.parent = proxy(parent)
        
    def __hash__(self):
        return id(self)
    
    def __cmp__(self, other):
        return cmp(self.label, other.label)

    def __str__(self):
        return self.label

    def __repr__(self):
        return "%s(%s)"%(self.__class__.__name__, `self.label`)
    
class Node(Base):
    def neighbors(self):
        return self.parent.neighbors(self)
    
class Edge(Base):
    pass



    
    
    
    
