"""Recursively traverse a molecule building up a canonical
representation.

Each vertex of a Molecule or Graph must have a attribute 'symorder' which
is a unique number.  This number guarantees only one traversal for
the graph.

Additionally each edge must have an attribute equiv_class which is
a unique value for each different type of bond.  This guarantees
proper canonicalization of edges as well as vertices.

usage
generateSymmetryOrders(graph)
"""

def _traverse(node, prevNode,
              visitedNodes, visitedEdges,
              nodes, edges):
    visitedNodes[node] = 1
    nodes.append(node)

    edgeIndex = 0
    edgesToTraverse = []

    for onode, edge in node.neighbors():        
        if prevNode is not None and onode is prevNode:
            # we are traversing back the way we came!
            # so don't...
            pass
        elif visitedNodes.has_key(onode):
            # a closure!
            # traverse.addClosure(node, onode, edge)
            edges.append(edge)
            visitedEdges[edge] = 1
        else:
            edgesToTraverse.append((onode.symorder,
                                    edge.equiv_class,
                                    edgeIndex,
                                    onode,
                                    edge))
        edgeIndex += 1

    if not edgesToTraverse:
        # dead end, return
        return

    edgesToTraverse.sort()

    for symorder, edgeEclass, index, onode, oedge in edgesToTraverse:
        if visitedNodes.has_key(onode):
            # somehow, we've seen this node so skip it
            continue
        edges.append(oedge)
        visitedEdges[oedge] = 1

        _traverse(onode,
                  node,
                  visitedNodes,
                  visitedEdges,
                  nodes,
                  edges
                  )

def _get_lowest_symorder(nodes):
    best = nodes[0]
    for node in nodes[1:]:
        if node.symorder < best.symorder:
            best = node
    return best


def generateSymmetryOrders(graph):
    """(graph) -> traverse the symmetry classes in order of the
    smallest edge equiv_classes to generate symmetry orders"""
    node = _get_lowest_symorder(graph.nodes())

    visitedNodes = {}
    visitedEdges = {}
    nodesUsed = []
    edgesUsed = []
    _traverse(node, None, visitedNodes, visitedEdges, nodesUsed, edgesUsed)
    i = 0
    # set the symmetry orders of the nodes and edges
    for n in nodesUsed:
        n.symorder = i
        i += 1

    i = 0
    for edge in edgesUsed:
        edge.symorder = i
        i += 1
        
