//......]*[.......]*[.......]*[.......]*[.......]*[.......]*[.......]*[.......].
// DpObject.cpp
//......]*[.......]*[.......]*[.......]*[.......]*[.......]*[.......]*[.......].
//
// Class functions for DpObject abstract class.
// The DpObject class defines the interface between the problem and the
// DpSolution class which is used to determine the solution.

#include "DpObject.hpp"

//......]*[.......]*[.......]*[.......]*[.......]*[.......]*[.......]*[.......].
//
// constructors * destructors
//
// Note: this class requires not memory management i.e. does not invoke
// new or delete.  Destructor is virtual since the class is abstract.
//
//......]*[.......]*[.......]*[.......]*[.......]*[.......]*[.......]*[.......].
//
//..............................................................................
//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//..............................................................................
//
// Default constructor
//
// Inputs:
//
// None.
//
// ABSTRACT: Initialize data member with values.
//
DpObject::DpObject(void)
{
   m_id = "";
   m_partner = -1;
   m_cost = 0;
}
//..............................................................................
//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//..............................................................................
//
// Copy constructor
//
// Inputs:
//
// DpObject dp - object to be copied.
//
// ABSTRACT: Create a copy of the input class.
//
DpObject::DpObject(const DpObject &dp)
{
   m_id      = dp.m_id;
   m_partner = dp.m_partner;
   m_cost    = dp.m_cost;
}
//..............................................................................
//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//..............................................................................
//
// Destructor
//
// Inputs:
//
// None.
//
// ABSTRACT: Class destructor - virtual
//
DpObject::~DpObject(void)
{
}
//......]*[.......]*[.......]*[.......]*[.......]*[.......]*[.......]*[.......].
//
// Class member functions
//
//..............................................................................
//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//..............................................................................
//
// Function: id
//
// Inputs:
//
// None.
//
// Returns:
//
// string - class identifier
//
// ABSTRACT: Return object identifier
//
string DpObject::id(void) const
{
   return m_id;
}
//..............................................................................
//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//..............................................................................
//
// Function: cost
//
// Inputs:
//
// None.
//
// Returns:
//
// double - cost associated with object
//
// ABSTRACT: Return object cost
//
double DpObject::cost(void) const
{
   return m_cost;
}
//..............................................................................
//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//..............................................................................
//
// Function: partner
//
// Inputs:
//
// None.
//
// Returns:
//
// int - index of partner object
//
// ABSTRACT: Return object partner
//
int DpObject::partner(void) const
{
   return m_partner;
}
//..............................................................................
//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//..............................................................................
//
// Function: clone
//
// Inputs:
//
// None.
//
// Returns:
//
// DpObject * - copy of object
//
// ABSTRACT: Return a copy of the object.  This function calls
// cloneDpObject which is a pure virtual function.
//
DpObject * DpObject::clone(void) const
{
   return cloneDpObject();
}
//..............................................................................
//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//..............................................................................
//
// Function: setCost
//
// Inputs:
//
// double cost - cost for object
//
// Returns:
//
// None.
//
// ABSTRACT: Set the cost associated with the object.
//
void DpObject::setCost(double cost)
{
   m_cost = cost;
}
//..............................................................................
//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//..............................................................................
//
// Function: setPartner
//
// Inputs:
//
// int index - index of partner
// int stage - strategy index
//
// Returns:
//
// None.
//
// ABSTRACT: Set the partner associated with the object.
//
void DpObject::setPartner(int index, int stage)
{
   m_partner = index;
}
//......]*[.......]*[.......]*[.......]*[.......]*[.......]*[.......]*[.......].
//
// Class friend functions
//
//..............................................................................
//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//..............................................................................
//
// Function: operator<<
//
// Inputs:
//
// ostream&  os - output stream to write to.
// DpObject& dp - object to write.
//
// Returns:
//
// ostream& - stream written to.
//
// ABSTRACT: Write object to output stream.  This function
// calls the print functions which is a pure virtual function.
//
ostream& operator<<( ostream& os, const DpObject& dp )
{
   dp.print(os);

   return(os);

}
//......]*[.......]*[.......]*[.......]*[.......]*[.......]*[.......]*[.......].
//      end       end       end       end       end       end       end
//......]*[.......]*[.......]*[.......]*[.......]*[.......]*[.......]*[.......].
