//......]*[.......]*[.......]*[.......]*[.......]*[.......]*[.......]*[.......].
// DpSolution.hpp
//......]*[.......]*[.......]*[.......]*[.......]*[.......]*[.......]*[.......].
//
// This class encapsulates the dynamic programming solution.  It uses the
// abstract class DpObject which contains the interface to the problem and
// uses the functions in this interface to calculate the solution to the
// problem. The class contains the following public functions:
//
// (1) DpSolution(const list<DpObject *> &dp, const int &N);
//
// Class constructor. Accepts a list of DpObjects that define
// the problem and the number of desired objects in the
// solution.
//
// (2) ~DpSolution(void)
//
// Class destructor.
//
// (3) list<DpObject *> solve(void);
//
// This functions returns the solution as a list of DpObjects.
//
// (4) DpObject * stage(int i,int j) const;
//
//  Returns the object at stage i and index j.  Used to examine the
//  solution field.
//
// (5) int stages(void) const;
//
//  Number of stages in the solution strategy.
//
// (6) int stages(int i) const;
//
//  Number of sub-stages in stage i of solution strategy.
//
// (7) friend ostream& operator<<( ostream& os, const DpSolution& dps );
//
//  print operator for printing the solution.
//
// The class data member m_stage holds the solution strategy.
//
//..............................................................................
//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//..............................................................................
#ifndef _DP_SOLUTION
#define _DP_SOLUTION

#include <list>
#include <vector>
#include <iostream>

#include "DpObject.hpp"

class DpSolution
{
   public:
      DpSolution(const list<DpObject *> &dp, const int &N);
     ~DpSolution(void);
      list<DpObject *> solve(void);
      int stages(void) const;
      int stages(int i) const;
      DpObject * stage(int i,int j) const;
      friend ostream& operator<<( ostream& os, const DpSolution& dps );
   private:
      typedef vector<DpObject *> Strategy;
      typedef vector<Strategy> Stage;
      Stage m_stage;  // solution strategy
};

#endif
//......]*[.......]*[.......]*[.......]*[.......]*[.......]*[.......]*[.......].
//      end       end       end       end       end       end       end
//......]*[.......]*[.......]*[.......]*[.......]*[.......]*[.......]*[.......].
