package com.mithra;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.InetAddress;
import java.net.Socket;
import java.security.MessageDigest;

import javax.crypto.Mac;
import javax.crypto.SecretKey;
import javax.crypto.spec.SecretKeySpec;

/**
 * This class will represent the client wanting to communicate with the Server.
 * @author Daniel Fremberg
 */
public class Client
{
    public static final int DEFAULT_PORT = 4711;
    
    public static void usage()
    {
        System.out.println("Usage: java Client <hostname> [<port>]");
        System.exit(0);
    }
    
    public static void main(String[] args)
    {
        int port = DEFAULT_PORT;
        Socket s = null;

        // Parse the port specification
        if ((args.length != 1) && (args.length != 2))
            usage();
        if (args.length == 1)
            port = DEFAULT_PORT;
        else
        {
            try
            {
                port = Integer.parseInt(args[1]);
            }
            catch (NumberFormatException e)
            {
                usage();
            }
        }

        try
        {
            // Create a socket to communicate to the specified host and port
            s = new Socket(args[0], port);

            ObjectOutputStream oos = new ObjectOutputStream(s.getOutputStream());
            ObjectInputStream ois = new ObjectInputStream(s.getInputStream());

            System.out.println("Connected to " + s.getInetAddress() + ":" + s.getPort());

            UserName userName = new UserName();
            userName.value = new String("Alice");

            oos.writeObject(userName);
            oos.flush();

            Object maybeNonce = ois.readObject();
            Nonce nonce = null;
            if (maybeNonce instanceof Nonce)
            {
                nonce = (Nonce) maybeNonce;
            }
            else
            {
                System.out.println("Error: Response should be a Nonce object.");
                System.exit(1);
            }

            System.out.println("Nonce: " + nonce.value);

            Mac mac = Mac.getInstance("HmacSHA1");
            MessageDigest md = MessageDigest.getInstance("SHA1");

    
            byte[] password = "secretPassword123456!".getBytes();            

            // Use a 4 byte salt value.
            byte[] salt = new byte[4];
            // Use the first and last 2 bytes of the password as salt.
            salt[0] = password[0];
            salt[1] = password[1];
            salt[2] = password[password.length - 1];
            salt[3] = password[password.length - 2]; 
            
            md.update(salt);            
            byte[] hashedPwd = md.digest(password);                   
            
            SecretKey macKey = new SecretKeySpec(hashedPwd, "Blowfish");
            mac.init(macKey);

            InetAddress remoteSocketAddress = s.getInetAddress();
            InetAddress localSocketAddress = s.getLocalAddress();

            mac.update(localSocketAddress.getAddress());
            mac.update(remoteSocketAddress.getAddress());
            mac.update(nonce.value);

            byte[] authenticatorValue = mac.doFinal();

            Authenticator authenticator = new Authenticator();
            authenticator.value = authenticatorValue;

            oos.writeObject(authenticator);
            oos.flush();
            
            System.out.println("Socket: " + s);

        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        // Always be sure to close the socket
        finally
        {
            try
            {
                if (s != null)
                    s.close();
            }
            catch (IOException e2)
            {
                ;
            }
        }
    }
}
