package com.mithra;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.InetAddress;
import java.net.Socket;
import java.security.MessageDigest;
import java.security.SecureRandom;
import java.util.HashMap;
import java.util.Map;

import javax.crypto.Mac;
import javax.crypto.SecretKey;
import javax.crypto.spec.SecretKeySpec;


/**
 * This class is the thread that handles all communication with a client.
 * @author Daniel Fremberg
 */
class Connection extends Thread
{
    protected Socket client;
    protected ObjectInputStream in;
    protected ObjectOutputStream out;
    protected SecureRandom random;
    protected Map passwordDatabase = new HashMap();

    // Initialize the streams and start the thread
    public Connection(Socket conn)
    {
        client = conn;

        try
        {
            // Disable Nagle's algorithm.
            client.setTcpNoDelay(true);
            in = new ObjectInputStream(client.getInputStream());
            out = new ObjectOutputStream(client.getOutputStream());
        }
        catch (IOException e)
        {
            try
            {
                client.close();
            }
            catch (IOException e2)
            {
                ;
            }
            System.err.println("Exception while getting socket streams: " + e);
            return;
        }

        MessageDigest md = null;
        System.out.print("Initializing randomizer..."); 
               
        try
        {
            random = SecureRandom.getInstance("SHA1PRNG");
            md = MessageDigest.getInstance("SHA1");            
        }
        catch (Exception e)
        {   
            e.printStackTrace();
            return;
        }
        System.out.println("done.");
        
        byte[] password = "secretPassword123456!".getBytes();

        // Use a 4 byte salt value.
        byte[] salt = new byte[4];
        // Use the first and last 2 bytes of the password as salt.
        salt[0] = password[0];
        salt[1] = password[1];
        salt[2] = password[password.length - 1];
        salt[3] = password[password.length - 2];        

        md.update(salt);
        byte[] hashedPwd = md.digest(password);
        
        passwordDatabase.put("Alice", hashedPwd);
        this.start();
    }

    public void run()
    {
        try
        {
            Object maybeUserName = in.readObject();
            UserName userName = null;
            if (maybeUserName instanceof UserName)
            {
                userName = (UserName) maybeUserName;   
            }
            else
            {
                System.out.println("Error: Object should be UserName");
                System.exit(-1);      
            }
            
            Nonce nonce = new Nonce();
            byte[] nonceValue = new byte[16];
            random.nextBytes(nonceValue);
            
            nonce.value = nonceValue;
            
            out.writeObject(nonce);            
            out.flush();
            
            Object maybeAuthenticator = in.readObject();
            Authenticator authenticator = null;
            if (maybeAuthenticator instanceof Authenticator)
            {
                authenticator = (Authenticator) maybeAuthenticator;
            }
            else
            {
                System.out.println("Error: Object should be Authenticator");
                System.exit(-1);      
            }
            
            Mac mac = Mac.getInstance("HmacSHA1");
            byte[] hashedPwd = (byte[]) passwordDatabase.get(userName.value);

            // Use "Blowfish" in order to make the key material go through unchanged (if not more than 448 bits). 
            // For example, using "DES" would truncate the key material to 64 bits and include 8 parity bits.
            SecretKey macKey = new SecretKeySpec(hashedPwd, "Blowfish");
            mac.init(macKey);

            InetAddress remoteSocketAddress = client.getInetAddress();
            InetAddress localSocketAddress = client.getLocalAddress();

            // Swapped order of addresses.
            mac.update(remoteSocketAddress.getAddress());
            mac.update(localSocketAddress.getAddress());
            mac.update(nonceValue);

            byte[] authenticatorValue = mac.doFinal();
            
            if (MessageDigest.isEqual(authenticator.value, authenticatorValue))
            {
                System.out.println("User: " + userName.value + " verified!");
            }
            else
            {
                System.out.println("User: " + userName.value + " is DENIED ACCESS!"); 
                System.out.println("Close socket!");                      
            }
            
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        finally
        {
            try
            {
                System.out.print("Closing socket...");
                client.close();
                System.out.println("done.");
            }
            catch (IOException e2)
            {
                ;
            }
        }
    }
}
