package com.mithra;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;

/**
 * This class will serve as the man-in-the-middle. 
 * @author Daniel Fremberg
 */
public class Mallory
{
    ServerSocket serverSocket;
    InetAddress realServerAddress;

    public Mallory (String server)
    {
        try
        {
            serverSocket = new ServerSocket(4711);
            realServerAddress = InetAddress.getByName(server);
        }
        catch (Exception e)
        {
            e.printStackTrace();
            System.exit(-1);
        }
    }

    public void launchManInTheMiddleAttack()
    {
        Socket clientSocket = null, realServerSocket = null;
        
        try
        {
            clientSocket = serverSocket.accept();
            
            System.out.println("Client connected to Mallory: " + clientSocket);
            
            realServerSocket = new Socket(realServerAddress, 4711);
            realServerSocket.setTcpNoDelay(true);
            System.out.println("Mallory connected to server: " + realServerSocket);
            
            ObjectOutputStream toRealServer = new ObjectOutputStream(realServerSocket.getOutputStream());
            ObjectInputStream fromRealServer = new ObjectInputStream(realServerSocket.getInputStream());
            
            ObjectInputStream fromClient = new ObjectInputStream(clientSocket.getInputStream());
            ObjectOutputStream toClient = new ObjectOutputStream(clientSocket.getOutputStream());
            
            // Begin relaying messages....
            
            // Relay UserName from client to real server
            toRealServer.writeObject(fromClient.readObject());
            toRealServer.flush();

            // Relay Nonce from real server to client
            toClient.writeObject(fromRealServer.readObject());
            toRealServer.flush();
            
            // Relay Authenticator from client to real server
            toRealServer.writeObject(fromClient.readObject());
            toRealServer.flush();            
            
            // The server should kick me out right about now. (Check the console of the server)
            // Due to TCP/IP buffering, this client will probably not be aware that it has been 
            // denied access until later sometime. Nevertheless, as far as the server is 
            // concerned, the connection to this client has been terminated.           
        }
        catch (Exception e)
        {   
            e.printStackTrace();
        }
        finally
        {
            try
            {
                clientSocket.close();
                realServerSocket.close();   
            }
            catch (IOException e)
            {
                e.printStackTrace();
            }
        }
    }

    public static void main (String[] args)
    {
        if (args.length == 0)
        {
            System.out.println("usage: java com.mithra.Mallory <real server's host name>");
            System.exit(0);
        }
        
        Mallory mallory = new Mallory(args[0]);
        mallory.launchManInTheMiddleAttack();
    }
}
