package com.mithra;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;


/**
 * This class will act as the server the clients want to talk to.
 * @author Daniel Fremberg, Cinnober Financial Technology AB
 */
public class Server extends Thread {
    public final static int DEFAULT_PORT = 4711;
    protected int port;
    protected ServerSocket listen_socket;
    
    // Exit with an error message.
    public static void fail(Exception e, String msg) {
        System.err.println(msg + ": " +  e);
        System.exit(1);
    }
    
    // Create a ServerSocket to listen for connections on and start the thread 
    // that will handle incoming connection requests.
    public Server(int port) {

        this.port = port;
        try 
        { 
            listen_socket = new ServerSocket(port); 
        }
        catch (IOException e) 
        { 
            fail(e, "Exception creating server socket");
        }
        System.out.println("Server: listening on port " + port);
        this.start();
    }
    
    // The body of the server thread.  Loop forever, listening for and
    // accepting connections from clients.  For each connection, 
    // create a Connection object to handle communication through the
    // new Socket.
    public void run() {
        try {
            while(true) {
                Socket client_socket = listen_socket.accept();
                System.out.println("New Connection: " + client_socket);
                new Connection(client_socket);
            }
        }
        catch (IOException e) { 
            fail(e, "Exception while listening for connections");
        }
    }
    
    // Start the server up, listening on an optionally specified port
    public static void main(String[] args) {
        
        new Server(DEFAULT_PORT);
    }
}

